--[[
PansyLabs Combat Indicator. Refer to LICENSE and README.md for details
]]--

if PL_COMBATINDICATOR then return end
local LAM = LibStub("LibAddonMenu-2.0")

PL_CombatIndicator = ZO_Object:Subclass()
PL_CombatIndicator.name = "PL_CombatIndicator"
PL_CombatIndicator.version = 2.00
-- Starting in 1.4, record the settings compat. version. The first settingsVersion was 1.33, as that was the 
-- production release at the time. Changing this forces players to re-do all the settings!
PL_CombatIndicator.settingsVersion = 2.00
PL_CombatIndicator.settingsDefaults = {
  ["enabled"] = true,
  ["enableSmallUiTweaks"] = false,
  ["colorCombat"] =  { 0.8, 0, 0, 1 },
  ["colorNormal"] = { 1, 1, 1, 1 },
}

function PL_CombatIndicator:New(...)
  local it = ZO_Object.New(self)
  it:Initialise()
  return it
end

function PL_CombatIndicator:Initialise()
  self.settings = ZO_SavedVars:New(PL_CombatIndicator.name .. "Settings", PL_CombatIndicator.settingsVersion, nil,
    PL_CombatIndicator.settingsDefaults, nil)
  self:InitAddonMenu()
  
  EVENT_MANAGER:RegisterForEvent(PL_CombatIndicator.name .. "OnPlayerActivated", EVENT_PLAYER_ACTIVATED, 
    function() self:OnPlayerActivatedOrDead() end)
  EVENT_MANAGER:RegisterForEvent(PL_CombatIndicator.name .. "OnPlayerCombatState", EVENT_PLAYER_COMBAT_STATE, 
    function(...) self:OnPlayerCombatState(...) end)
  EVENT_MANAGER:RegisterForEvent(PL_CombatIndicator.name .. "OnPlayerDead", EVENT_PLAYER_DEAD, 
  function() self:OnPlayerActivatedOrDead() end)
end

function PL_CombatIndicator:OnPlayerActivatedOrDead()
  if (not self.settings.enabled) then return end
  self:SetCombat(false)
end

function PL_CombatIndicator:OnPlayerCombatState(_, inCombat)
  if (not self.settings.enabled) then return end
  self:SetCombat(inCombat) 
end

-- Used in response to settings changes.
function PL_CombatIndicator:UpdateForCurrentStatus()
  if not self.settings.enabled then return end
  self:SetCombat(IsUnitInCombat("player"))
end

function PL_CombatIndicator:SetCombat(inCombat)
  PL_CombatIndicator:SetCompassColor(inCombat and self.settings.colorCombat or self.settings.colorNormal)
  -- We can make some small optimisations to the inCombat indicator so it is clear at small UI sizes.
  local alpha = inCombat and 0.4 or 1
  ZO_CompassFrameCenterBottomMungeOverlay:SetAlpha(alpha)    
  ZO_CompassFrameCenterTopMungeOverlay:SetAlpha(alpha)    
  local desaturation = inCombat and (self.settings.enableSmallUiTweaks and -0.8 or -0.2) or 0
  ZO_CompassFrameCenter:SetDesaturation(desaturation)
  ZO_CompassFrameLeft:SetDesaturation(desaturation)
  ZO_CompassFrameRight:SetDesaturation(desaturation)
end

function PL_CombatIndicator:SetCompassColor(col)
  if not col then return end
  ZO_CompassFrameLeft:SetColor(col[1], col[2], col[3], col[4])
  ZO_CompassFrameCenter:SetColor(col[1], col[2], col[3], col[4])
  ZO_CompassFrameRight:SetColor(col[1], col[2], col[3], col[4])
  -- Also change the colour of "area" indicators (normally green/blue).
  -- GetNumChildren seems to return larger than expected counts.
  for i = 1,_G["ZO_CompassContainer"]:GetNumChildren() do
    local areaTexture = _G["ZO_CompassAreaTexture" .. i]
    if areaTexture then
      areaTexture.left:SetColor(col[1], col[2], col[3], col[4])
      areaTexture.center:SetColor(col[1], col[2], col[3], col[4])    
      areaTexture.right:SetColor(col[1], col[2], col[3], col[4])  
    end
  end
end

function PL_CombatIndicator:InitAddonMenu()
  LAM:RegisterAddonPanel("PL_CombatIndicatorPanel", {
    type = "panel",
    name = GetString(PL_COMBATINDICATOR_SETTINGS_TITLE),
    version = PL_CombatIndicator.version,
    author = "@iyoro (EU)",
    registerForRefresh = true,
  })  
  LAM:RegisterOptionControls("PL_CombatIndicatorPanel", {
    [1] = {
      type = "checkbox",
      name = GetString(PL_COMBATINDICATOR_SETTING_LABEL_ENABLE),
      tooltip = GetString(PL_COMBATINDICATOR_SETTING_TOOLTIP_ENABLE),
      getFunc = function() return self.settings.enabled end,
      setFunc = function(value) 
        self.settings.enabled = value 
        if value then 
          self:UpdateForCurrentStatus()
        else
          self:SetCompassColor({1, 1, 1, 1})
        end
      end,
    },  
    [2] = {
      type = "colorpicker",
      name = GetString(PL_COMBATINDICATOR_SETTING_LABEL_COLOR_NORMAL),
      tooltip = GetString(PL_COMBATINDICATOR_SETTING_TOOLTIP_COLOR_NORMAL),
      getFunc = function() return unpack(self.settings.colorNormal) end,
      setFunc = function(r, g, b, a) 
        self.settings.colorNormal = {r, g, b, a} 
        self:UpdateForCurrentStatus()
      end
    },
    [3] = {
      type = "colorpicker",
      name = GetString(PL_COMBATINDICATOR_SETTING_LABEL_COLOR_COMBAT),
      tooltip = GetString(PL_COMBATINDICATOR_SETTING_TOOLTIP_COLOR_COMBAT),
      getFunc = function() return unpack(self.settings.colorCombat) end,
      setFunc = function(r, g, b, a)
        self.settings.colorCombat = {r, g, b, a}
        self:UpdateForCurrentStatus()
      end
    },
    [4] = {
      type = "checkbox",
      name = GetString(PL_COMBATINDICATOR_SETTING_LABEL_OPTIMISE_FOR_UI_SCALE),
      tooltip = GetString(PL_COMBATINDICATOR_SETTING_TOOLTIP_OPTIMISE_FOR_UI_SCALE),
      getFunc = function() return self.settings.enableSmallUiTweaks end,
      setFunc = function(value) 
        self.settings.enableSmallUiTweaks = value 
        self:UpdateForCurrentStatus()
      end,
    },
  })
end

EVENT_MANAGER:RegisterForEvent(PL_CombatIndicator.name .. "OnAddOnLoaded", EVENT_ADD_ON_LOADED, function(_, addonName)
  if (addonName == PL_CombatIndicator.name) then
    PL_COMBATINDICATOR = PL_CombatIndicator:New()
  end
end)
